/**
 * Description:
 * 
 * Servlet implementation showcase. 
 * See also the files README.txt, web.xml, build.xml
 *          log4j.configuration, index.html       
 * 
 * Franz Schenk <schenk@informatik.uni-goettingen.de>
 * 
 */
package org.dbis.xmlpraktikum;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.io.PrintWriter;
import java.net.HttpURLConnection;
import java.net.URL;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.log4j.Logger;


/**
 * Example Servlet 
 *   for the XML Praktikum
 */


public class ServletExample 
	extends javax.servlet.http.HttpServlet
	implements javax.servlet.Servlet{

	private static final long serialVersionUID = 1L;
	/* 
	 * Servlet initialisation sequence: 
	 *  - Class Constructor
	 *  - init() 
	 */
	public ServletExample(){
		super();
		logger.info("servlet constructor");
	}
	
	public void init() throws ServletException {
		logger.info("servlet init");
	}

	/* Variables and servlet instances

	 * 	 Every time, the servlet is called 
	 *   ( e.g. invocing  doGet() by a http request to 
	 *     http://localhost:8080/servletdemo/demo)
	 *   a servlet instance is created. 
	 *   
	 *   These instances share class variables 
	 *       (like the logger in this example).
	 *       
	 *   If the class declares further variables, they
	 *   are also shared if they become defined during 
	 *   the initialisation process.  
	 *   
	 *   If however a variable is defined within a method
	 *   like doGet() it is only visible locally. 
	 *      
	 */

	/** log4j-logger */
	/* configuration of the logger is done via log4j.properties
	 * file in the WEB-INF/classes directory of the servlet
	 * destination for the log-file is $CATALINA_HOME/logs */ 
	private static Logger logger = Logger.getLogger(ServletExample.class);

	
	/**
	 * receive and answer a http connection 
	 * 
	 * POST is intended to be used when sending information to
	 *      a servlet. The information is the content of the message
	 *      (as opposed to the GET method where information has to
	 *       be encoded into the URL)
	 */
	protected void doPost(HttpServletRequest request,
			HttpServletResponse response) throws ServletException, IOException {
		logger.info("doPost invoked");

		
		StringBuffer result = new StringBuffer();
		result.append("<hr/><h1>received via http-Post the following content</h1>");
		String line;
		BufferedReader buf_in = new BufferedReader(new InputStreamReader(
				request.getInputStream()));
		while ((line = buf_in.readLine()) != null) {
			result.append(line + "\n");
		}
		buf_in.close();
		result.append("<hr/>");
		
		// return an answer 
		logger.info("sending response...");
		PrintWriter out = response.getWriter();
		out.println(result.toString());
		logger.info("...finished");
	}
	
	
	
	/** doGet: direct access, e.g. via webbrowser 
	 * 
	 * GET is intended to fetch(get) information from a servlet.
	 *     parameters (and thus also information) can be encoded into
	 *     the URL, e.g.
	 *     http://localhost:8080/servletdemo/demo?demotext=MyDemoTextInTheTextArea&inputaction=send+directly
	 * */
	protected void doGet(HttpServletRequest req,
			HttpServletResponse resp) throws ServletException, IOException {

		logger.info("doGet invoked.");
		resp.setContentType("text/html");
		PrintWriter out = resp.getWriter();
		StringBuffer result = new StringBuffer();

		/* retrieve the request parameter 'demotext'
		 * (encoded in the URL from the text area content of the 
		 *  HTML form in index.html) */
		String demotext = req.getParameter("demotext");
		if (demotext == null){
			logger.error("no text received");
			result.append(createDocumentHeader("Error"));
			result.append("<h1>No text input</h1>");
		}
		else{
			logger.info("text received: "+demotext);

			/* test for the request parameter 'inputaction' 
			 *   (which is set by the submit button 'SEND DIRECTLY')*/
			String action = req.getParameter("inputaction");

			/*  simply display the contents of the text area */
			if (action != null){
				logger.info("InputAction: "+action);
				result.append(createDocumentHeader("Demo Servlet"));
				result.append("<h1>Received Text:</h1>");
				result.append(demotext);
			}
			else{

				/* test for the request parameter 'inputaction2' 
				 *  (which is set by the submit button 'SEND A HTTP MESSAGE')*/
				action = req.getParameter("inputaction2");

				/*    This example shows how a message can be sent via http:
				 *      create a URLConnection to a service 
				 *      send via outputstream
				 *      receive any results via the inputstream
				 *    
				 *    In this example, the connection is established
				 *    to the same servlet at http://localhost:8080/servletdemo/demo .
				 *    
				 *    The doPost method returns the content of the message (pass-through).
				 *    
				 *    This is only an example on how to establish and
				 *    use a http-connection!!
				 *    
				 *    EXERCISE:
				 *      create a copy of this servlet and deploy it under a different name
				 *      e.g. servletdemo2, maybe even on a different server.
				 *      Let the two servlets communicate: send the message from one servlet 
				 *      to the other.
				 */

				if (action != null){
					logger.info("InputAction2: "+action);
					try {

						/* this is the URL of the destination servlet. 
						 * 
						 * Try to send the message to a different servlet and process
						 * the service call.
						 */
						URL url = new URL("http://localhost:8080/servletdemo/demo");
						HttpURLConnection urlcon = (HttpURLConnection) url.openConnection();

						// Setting some connection parameters
						urlcon.setRequestMethod("POST");
						urlcon.setDoInput(true);
						urlcon.setDoOutput(true);
						urlcon.setUseCaches(false);
						urlcon.setRequestProperty("CONTENT_LENGTH", ""
								+ demotext.length());
						urlcon.setRequestProperty("Content-Type",
						"application/x-www-form-urlencoded");

						// Sending the request-content via outputstream
						OutputStreamWriter osw = new OutputStreamWriter(urlcon
								.getOutputStream());
						osw.write(demotext);
						osw.flush();
						osw.close();
						logger.info("### message sent.");

						// Processing the answer to the http-request
						result.append("<h3>Text sent, result was:</h3>");
						String line;
						StringBuffer buf = new StringBuffer();
						BufferedReader buf_in = new BufferedReader(new InputStreamReader(
								urlcon.getInputStream()));
						while ((line = buf_in.readLine()) != null) {
							buf.append(line + "\n");
						}
						buf_in.close();
						urlcon.disconnect();
						result.append(buf);
						logger.info("received response to request:" + result);
					} catch (IOException e) {
						logger.fatal(e.getMessage());
						result.append(e.getMessage());
					} catch (Throwable th) {
						logger.fatal(th.getMessage());
						result.append(th.getMessage());
					}
				}
				else{
					logger.error("No Action");
					result.append(createDocumentHeader("Demo"));
					result.append("<h1>XML Praktikum</h1><h3>Demo Page</h3>");
				}
			}
		}
		out.println(result.append(createDocumentFooter()).toString());
	}
		
	private String createDocumentHeader(String title){
		return "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0 "
		+ "Transitional//EN\">\n"
		+ "<HTML>\n"
		+ "<HEAD><TITLE>"
		+ title
		+ "</TITLE></HEAD>\n"
		+" <BODY>\n" ;
	}
	private String createDocumentFooter(){
		return "</BODY></HTML>\n";
	}
}

